@@ -1,4 +1,4 @@
-# Copyright 2021 The Flax Authors.
+# Copyright 2022 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@
 from collections.abc import Iterable  # pylint: disable=g-importing-member
 from flax import struct
 from flax.core import Scope
-from flax.deprecated.nn import initializers
+from flax.linen import initializers
 from jax import lax
 
 import jax.numpy as jnp
