@@ -23,10 +23,19 @@ from .activation import (
   elu as elu,
   gelu as gelu,
   glu as glu,
+  hard_sigmoid as hard_sigmoid,
+  hard_silu as hard_silu,
+  hard_swish as hard_swish,
+  hard_tanh as hard_tanh,
   leaky_relu as leaky_relu,
   log_sigmoid as log_sigmoid,
   log_softmax as log_softmax,
+  logsumexp as logsumexp,
+  normalize as normalize,
+  one_hot as one_hot,
   relu as relu,
+  relu6 as relu6,
+  selu as selu,
   sigmoid as sigmoid,
   silu as silu,
   soft_sign as soft_sign,
