@@ -12,11 +12,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""Initializers for Flax.
-"""
+"""Initializers for Flax."""
 
 # pylint: disable=unused-import
 # re-export initializer functions from jax.nn
+from jax.nn.initializers import constant
+from jax.nn.initializers import delta_orthogonal
 from jax.nn.initializers import glorot_normal
 from jax.nn.initializers import glorot_uniform
 from jax.nn.initializers import he_normal
@@ -28,7 +29,6 @@ from jax.nn.initializers import lecun_uniform
 from jax.nn.initializers import normal
 from jax.nn.initializers import ones
 from jax.nn.initializers import orthogonal
-from jax.nn.initializers import delta_orthogonal
 from jax.nn.initializers import uniform
 from jax.nn.initializers import variance_scaling
 from jax.nn.initializers import xavier_normal
