@@ -14,29 +14,30 @@
 
 """Normalization modules for Flax."""
 
-from typing import (Any, Callable, Optional, Tuple, Iterable, Union)
+from typing import (Any, Callable, Iterable, Optional, Tuple, Union)
+from flax.linen.dtypes import canonicalize_dtype
 
+from flax.linen.module import Module, compact, merge_param  # pylint: disable=g-multiple-import
 from jax import lax
 from jax.nn import initializers
 import jax.numpy as jnp
 
-from flax.linen.module import Module, compact, merge_param
-
 
 PRNGKey = Any
 Array = Any
-Shape = Tuple[int]
+Shape = Tuple[int, ...]
 Dtype = Any  # this could be a real type?
 
 Axes = Union[int, Iterable[int]]
 
 
-def _canonicalize_axes(rank: int, axes: Axes) -> Iterable[int]:
+def _canonicalize_axes(rank: int, axes: Axes) -> Tuple[int, ...]:
   """Returns a tuple of deduplicated, sorted, and positive axes."""
   if not isinstance(axes, Iterable):
     axes = (axes,)
   return tuple(set([rank + axis if axis < 0 else axis for axis in axes]))
 
+
 def _abs_sq(x):
   """Computes the elementwise square of the absolute value |x|^2."""
   if jnp.iscomplexobj(x):
@@ -46,22 +47,38 @@ def _abs_sq(x):
 
 
 def _compute_stats(x: Array, axes: Axes,
+                   dtype: Optional[Dtype],
                    axis_name: Optional[str] = None,
                    axis_index_groups: Any = None):
   """Computes mean and variance statistics.
 
   This implementation takes care of a few important details:
-  - Computes in float32 precision for half precision inputs
-  -  mean and variance is computable in a single XLA fusion,
+  - Computes in float32 precision for stability in half precision training.
+  - mean and variance are computable in a single XLA fusion,
     by using Var = E[|x|^2] - |E[x]|^2 instead of Var = E[|x - E[x]|^2]).
   - Clips negative variances to zero which can happen due to
     roundoff errors. This avoids downstream NaNs.
   - Supports averaging across a parallel axis and subgroups of a parallel axis
     with a single `lax.pmean` call to avoid latency.
+
+  Arguments:
+    x: Input array.
+    axes: The axes in ``x`` to compute mean and variance statistics for.
+    dtype: Optional dtype specifying the minimal precision. Statistics
+      are always at least float32 for stability (default: dtype of x).
+    axis_name: Optional name for the pmapped axis to compute mean over.
+    axis_index_groups: Optional axis indices.
+
+  Returns:
+    A pair ``(mean, var)``.
   """
+  if dtype is None:
+    dtype = jnp.result_type(x)
   # promote x to at least float32, this avoids half precision computation
   # but preserves double or complex floating points
-  x = jnp.asarray(x, jnp.promote_types(jnp.float32, jnp.result_type(x)))
+  dtype = jnp.promote_types(dtype, jnp.float32)
+  x = jnp.asarray(x, dtype)
+  
   mean = jnp.mean(x, axes)
   mean2 = jnp.mean(_abs_sq(x), axes)
   if axis_name is not None:
@@ -86,7 +103,25 @@ def _normalize(mdl: Module, x: Array, mean: Array, var: Array,
                scale_init: Callable[[PRNGKey, Shape, Dtype], Array]):
   """"Normalizes the input of a normalization layer and optionally applies a learned scale and bias.
 
-  A seperate bias and scale is learned for each feature as specified by feature_axes.
+  Arguments:
+    mdl: Module to apply the normalization in (normalization params will reside
+      in this module).
+    x: The input.
+    mean: Mean to use for normalization.
+    var: Variance to use for normalization.
+    reduction_axes: The axes in ``x`` to reduce.
+    feature_axes: Axes containing features. A separate bias and scale is learned
+      for each specified feature.
+    dtype: The dtype of the result (default: infer from input and params).
+    param_dtype: The dtype of the parameters.
+    epsilon: Normalization epsilon.
+    use_bias: If true, add a bias term to the output.
+    use_scale: If true, scale the output.
+    bias_init: Initialization function for the bias term.
+    scale_init: Initialization function for the scaling function.
+
+  Returns:
+    The normalized input.
   """
   reduction_axes = _canonicalize_axes(x.ndim, reduction_axes)
   feature_axes = _canonicalize_axes(x.ndim, feature_axes)
@@ -102,15 +137,19 @@ def _normalize(mdl: Module, x: Array, mean: Array, var: Array,
     reduced_feature_shape.append(x.shape[ax])
   y = x - mean
   mul = lax.rsqrt(var + epsilon)
+  args = [x]
   if use_scale:
     scale = mdl.param('scale', scale_init, reduced_feature_shape,
                       param_dtype).reshape(feature_shape)
     mul *= scale
+    args.append(scale)
   y *= mul
   if use_bias:
     bias = mdl.param('bias', bias_init, reduced_feature_shape,
                      param_dtype).reshape(feature_shape)
     y += bias
+    args.append(bias)
+  dtype = canonicalize_dtype(*args, dtype=dtype)
   return jnp.asarray(y, dtype)
 
 
@@ -151,7 +190,7 @@ class BatchNorm(Module):
     momentum: decay rate for the exponential moving average of
       the batch statistics.
     epsilon: a small float added to variance to avoid dividing by zero.
-    dtype: the dtype of the computation (default: float32).
+    dtype: the dtype of the result (default: infer from input and params).
     param_dtype: the dtype passed to parameter initializers (default: float32).
     use_bias:  if True, bias (beta) is added.
     use_scale: if True, multiply by scale (gamma).
@@ -171,7 +210,7 @@ class BatchNorm(Module):
   axis: int = -1
   momentum: float = 0.99
   epsilon: float = 1e-5
-  dtype: Dtype = jnp.float32
+  dtype: Optional[Dtype] = None
   param_dtype: Dtype = jnp.float32
   use_bias: bool = True
   use_scale: bool = True
@@ -221,11 +260,13 @@ class BatchNorm(Module):
     else:
       mean, var = _compute_stats(
           x, reduction_axes,
+          dtype=self.dtype,
           axis_name=self.axis_name if not initializing else None,
           axis_index_groups=self.axis_index_groups)
 
       if not initializing:
-        ra_mean.value = self.momentum * ra_mean.value + (1 - self.momentum) * mean
+        ra_mean.value = self.momentum * ra_mean.value + (1 -
+                                                         self.momentum) * mean
         ra_var.value = self.momentum * ra_var.value + (1 - self.momentum) * var
 
     return _normalize(
@@ -237,6 +278,7 @@ class BatchNorm(Module):
 
 class LayerNorm(Module):
   """Layer normalization (https://arxiv.org/abs/1607.06450).
+
   Operates on the last axis of the input data.
 
   It normalizes the activations of the layer for each given example in a
@@ -246,7 +288,7 @@ class LayerNorm(Module):
 
   Attributes:
     epsilon: A small float added to variance to avoid dividing by zero.
-    dtype: the dtype of the computation (default: float32).
+    dtype: the dtype of the result (default: infer from input and params).
     param_dtype: the dtype passed to parameter initializers (default: float32).
     use_bias:  If True, bias (beta) is added.
     use_scale: If True, multiply by scale (gamma). When the next layer is linear
@@ -256,7 +298,7 @@ class LayerNorm(Module):
     scale_init: Initializer for scale, by default, one.
   """
   epsilon: float = 1e-6
-  dtype: Any = jnp.float32
+  dtype: Optional[Dtype] = None
   param_dtype: Dtype = jnp.float32
   use_bias: bool = True
   use_scale: bool = True
@@ -276,8 +318,8 @@ class LayerNorm(Module):
     reduction_axes = (-1,)
     feature_axes = (-1,)
 
-    # TODO suport axis_name for model parallelism?
-    mean, var = _compute_stats(x, reduction_axes, None, None)
+    # TODO(jheek) suport axis_name for model parallelism?
+    mean, var = _compute_stats(x, reduction_axes, self.dtype, None, None)
 
     return _normalize(
         self, x, mean, var, reduction_axes, feature_axes,
@@ -301,19 +343,19 @@ class GroupNorm(Module):
         proposed by the original group normalization paper.
       group_size: the number of channels in a group.
       epsilon: A small float added to variance to avoid dividing by zero.
-      dtype: the dtype of the computation (default: float32).
+      dtype: the dtype of the result (default: infer from input and params).
       param_dtype: the dtype passed to parameter initializers (default: float32).
       use_bias:  If True, bias (beta) is added.
-      use_scale: If True, multiply by scale (gamma). When the next layer is linear
-        (also e.g. nn.relu), this can be disabled since the scaling will be done
-        by the next layer.
+      use_scale: If True, multiply by scale (gamma). When the next layer is
+        linear (also e.g. nn.relu), this can be disabled since the scaling will
+        be done by the next layer.
       bias_init: Initializer for bias, by default, zero.
       scale_init: Initializer for scale, by default, one.
   """
   num_groups: Optional[int] = 32
   group_size: Optional[int] = None
   epsilon: float = 1e-6
-  dtype: Any = jnp.float32
+  dtype: Optional[Dtype] = None
   param_dtype: Dtype = jnp.float32
   use_bias: bool = True
   use_scale: bool = True
@@ -338,8 +380,9 @@ class GroupNorm(Module):
     if ((self.num_groups is None and self.group_size is None) or
         (self.num_groups is not None and self.group_size is not None)):
       raise ValueError('Either `num_groups` or `group_size` should be '
-                       'specified, but not both of them.')
-    num_groups = self.num_groups
+                       'specified. If `group_size` is to be specified, '
+                       'pass `num_groups=None` as argument to override '
+                       'the default `num_groups` value of 32.')
 
     channels = x.shape[-1]
     if self.group_size is not None:
@@ -347,6 +390,9 @@ class GroupNorm(Module):
         raise ValueError('Number of channels ({}) is not multiple of the '
                          'group size ({}).'.format(channels, self.group_size))
       num_groups = channels // self.group_size
+    else:
+      num_groups = self.num_groups
+      assert isinstance(num_groups, int)
 
     if num_groups <= 0 or channels % num_groups != 0:
       raise ValueError('Number of groups ({}) does not divide the number'
@@ -356,11 +402,13 @@ class GroupNorm(Module):
     group_shape = x.shape[:-1] + (num_groups, group_size)
 
     def broadcast_stat(stat):
-      stat = jnp.broadcast_to(stat[..., None], (x.shape[0], num_groups, group_size))
+      stat = jnp.broadcast_to(stat[..., None],
+                              (x.shape[0], num_groups, group_size))
       return stat.reshape((x.shape[0], num_groups * group_size))
 
-    # TODO suport axis_name for model parallelism?
-    mean, var = _compute_stats(x.reshape(group_shape), reduction_axes, None, None)
+    # TODO(jheek): suport axis_name for model parallelism?
+    mean, var = _compute_stats(
+        x.reshape(group_shape), reduction_axes, self.dtype, None, None)
     mean = broadcast_stat(mean)
     var = broadcast_stat(var)
 
